@@ -2,7 +2,7 @@
  *
  *  DHCP client library with GLib integration
  *
- *  Copyright (C) 2009-2011  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2009-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -1126,7 +1126,6 @@ static int ipv4ll_recv_arp_packet(GDHCPClient *dhcp_client)
 	int target_conflict;
 
 	memset(&arp, 0, sizeof(arp));
-	bytes = 0;
 	bytes = read(dhcp_client->listener_sockfd, &arp, sizeof(arp));
 	if (bytes < 0)
 		return bytes;
@@ -1362,6 +1361,7 @@ static void restart_dhcp(GDHCPClient *dhcp_client, int retry_times)
 
 	dhcp_client->retry_times = retry_times;
 	dhcp_client->requested_ip = 0;
+	dhcp_client->state = INIT_SELECTING;
 	switch_listening_mode(dhcp_client, L2);
 
 	g_dhcp_client_start(dhcp_client, dhcp_client->last_address);
@@ -1596,6 +1596,9 @@ static GList *get_addresses(GDHCPClient *dhcp_client,
 	uint8_t *option;
 	char *str;
 
+	if (value == NULL || len < 4)
+		return NULL;
+
 	iaid = get_uint32(&value[0]);
 	if (dhcp_client->iaid != iaid)
 		return NULL;
@@ -1663,7 +1666,7 @@ static GList *get_addresses(GDHCPClient *dhcp_client,
 
 	if (addr_count > 0 && st == 0) {
 		/* We only support one address atm */
-		char str[INET6_ADDRSTRLEN + 1];
+		char addr_str[INET6_ADDRSTRLEN + 1];
 
 		if (preferred > valid)
 			/* RFC 3315, 22.6 */
@@ -1672,11 +1675,11 @@ static GList *get_addresses(GDHCPClient *dhcp_client,
 		dhcp_client->T1 = T1;
 		dhcp_client->T2 = T2;
 
-		inet_ntop(AF_INET6, &addr, str, INET6_ADDRSTRLEN);
+		inet_ntop(AF_INET6, &addr, addr_str, INET6_ADDRSTRLEN);
 		debug(dhcp_client, "count %d addr %s T1 %u T2 %u",
-			addr_count, str, T1, T2);
+			addr_count, addr_str, T1, T2);
 
-		list = g_list_append(list, g_strdup(str));
+		list = g_list_append(list, g_strdup(addr_str));
 
 		if (code == G_DHCPV6_IA_NA)
 			memcpy(&dhcp_client->ia_na, &addr,
@@ -1700,6 +1703,9 @@ static GList *get_dhcpv6_option_value_list(GDHCPClient *dhcp_client,
 	char *str;
 	int i;
 
+	if (value == NULL)
+		return NULL;
+
 	switch (code) {
 	case G_DHCPV6_DNS_SERVERS:	/* RFC 3646, chapter 3 */
 	case G_DHCPV6_SNTP_SERVERS:	/* RFC 4075, chapter 4 */
@@ -1847,7 +1853,7 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 			re = dhcp_recv_l3_packet(&packet,
 						dhcp_client->listener_sockfd);
 	} else if (dhcp_client->listen_mode == L_ARP) {
-		re = ipv4ll_recv_arp_packet(dhcp_client);
+		ipv4ll_recv_arp_packet(dhcp_client);
 		return TRUE;
 	}
 	else
@@ -1860,6 +1866,9 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 		return TRUE;
 
 	if (dhcp_client->type == G_DHCP_IPV6) {
+		if (packet6 == NULL)
+			return TRUE;
+
 		count = 0;
 		client_id = dhcpv6_get_option(packet6, pkt_len,
 				G_DHCPV6_CLIENTID, &option_len,	&count);
@@ -1892,8 +1901,11 @@ static gboolean listener_event(GIOChannel *channel, GIOCondition condition,
 		} else
 			dhcp_client->status_code = 0;
 
-	} else
+	} else {
 		message_type = dhcp_get_option(&packet, DHCP_MESSAGE_TYPE);
+		if (message_type == NULL)
+			return TRUE;
+	}
 
 	if (message_type == NULL && client_id == NULL)
 		/* No message type / client id option, ignore package */
